/**
 * Default connection handler to API.
 */
import ConnectionRest from "@/api/connection/rest";
import ConnectionWebSocket from "@/api/connection/socket";
import ServiceFactoryReset from "@/api/services/factoryReset";
import type ServiceConfig from "@/api/services/config";
import type ServiceAuth from "@/api/services/authenticate";
import type ServiceInterface from "@/api/services/ServiceInterface";
import type ConnectionInterface from "@/api/connection/ConnectionInterface";
import type ServiceProfiles from "@/api/services/profiles";
import type ServiceActivities from "@/api/services/activities";
import type ServiceMacros from "@/api/services/macros";
import type ServiceIntegrations from "@/api/services/integrations";
import type ServiceRemotes from "@/api/services/remotes";
import type ServiceIr from "@/api/services/ir";
import type ServiceResources from "@/api/services/resources";
import type ServiceDocks from "@/api/services/docks";
import type ServiceLogs from "@/api/services/logs";

export type ConnectionSetup = {
  baseUrl: string;
  user: string;
};

export default class Connection implements ConnectionInterface {
  /**
   * API host.
   * @protected
   */
  protected _host: string;
  protected _config: ConnectionSetup;
  protected _rest: ConnectionRest;
  protected _websocket: ConnectionWebSocket;

  protected _services: Map<string, ServiceInterface>;

  constructor(config: ConnectionSetup) {
    this._config = { ...config };
    this._host = this._config.baseUrl;
    this._rest = new ConnectionRest(config);
    this._websocket = new ConnectionWebSocket(config);
    this._services = new Map<string, ServiceInterface>();
  }

  rest() {
    return this._rest;
  }

  websocket() {
    return this._websocket;
  }

  addService(id: string, service: ServiceInterface) {
    service.setConnection(this);
    this._services.set(id, service);
    return this;
  }

  getService(id: string): undefined | ServiceInterface {
    return this._services.get(id);
  }

  get auth(): ServiceAuth {
    return this.getService("auth") as ServiceAuth;
  }

  get config(): ServiceConfig {
    return this.getService("config") as ServiceConfig;
  }

  get profiles(): ServiceProfiles {
    return this.getService("profiles") as ServiceProfiles;
  }

  get activities(): ServiceActivities {
    return this.getService("activities") as ServiceActivities;
  }

  get macros(): ServiceMacros {
    return this.getService("macros") as ServiceMacros;
  }

  get integrations(): ServiceIntegrations {
    return this.getService("intg") as ServiceIntegrations;
  }

  get remotes(): ServiceRemotes {
    return this.getService("remotes") as ServiceRemotes;
  }

  get ir(): ServiceIr {
    return this.getService("ir") as ServiceIr;
  }

  get resources(): ServiceResources {
    return this.getService("resources") as ServiceResources;
  }

  get docks(): ServiceDocks {
    return this.getService("docks") as ServiceDocks;
  }

  get logs(): ServiceLogs {
    return this.getService("logs") as ServiceLogs;
  }

  getFactoryReset(): ServiceFactoryReset {
    const id = "factory_reset";
    if (!this._services.has(id)) {
      this.addService(id, new ServiceFactoryReset(this));
    }
    return this.getService(id) as ServiceFactoryReset;
  }
}
