import type PingerInterface from "@/api/connection/PingerInterface";
import ApiConnection, { ApiPinger } from "@/api";

export type ConnectionStateChange = (state: boolean | null) => void;

export default class PingApi {
  protected _pinger;
  protected timer: any;
  protected currentState: boolean | null = null;
  protected progress = false;
  protected callbacks: Map<
    string,
    { callback: ConnectionStateChange; always: boolean }
  > = new Map();
  protected failed = 0;
  protected readonly failedLimit = 3;

  constructor(service: PingerInterface) {
    this._pinger = service;
    this.ping();
  }

  static useSocket() {
    ApiConnection.websocket().connect();
    ApiPinger.setPinger(ApiConnection.websocket());
  }
  static useRest() {
    ApiPinger.setPinger(ApiConnection.rest());
    ApiConnection.websocket().disconnect();
  }

  setPinger(service: PingerInterface) {
    if (this._pinger?.name === service.name) {
      return;
    }

    this.progress = false;
    clearInterval(this.timer);
    this._pinger = service;
    this.ping();
  }

  ping() {
    this.doPing();
    this.timer = setInterval(() => {
      this.doPing();
    }, this._pinger.pingTimeout);
  }

  protected doPing() {
    if (this.progress) {
      return;
    }
    this.progress = true;
    this._pinger
      .ping()
      .then((result) => {
        this.setCurrentState(result);
      })
      .catch(() => {
        this.setCurrentState(false);
      });
  }

  protected setCurrentState(state: boolean) {
    this.progress = false;
    if (!state) {
      this.failed++;

      if (this.failed < this.failedLimit) {
        return;
      }
    }

    this.failed = 0;

    this.currentState = state;
    this.callbacks.forEach((item) => {
      if (this.currentState === state && !item.always) {
        return;
      }
      item.callback(this.currentState);
    });
  }

  addStateChangeCallback(
    name: string,
    callback: ConnectionStateChange,
    replace = false,
    always = false
  ) {
    if (this.callbacks.has(name) && !replace) {
      throw `Connection state change callback with name: "${name}" already exists`;
    }
    this.callbacks.set(name, { callback, always });
  }
}
