import type {
  WsConnectedCallback,
  WsDisconnectedCallback,
  WsErrorCallback,
  WsMessageCallback,
  WsStateChange,
} from "@/types/websocket";
import type { ConnectionSetup } from "@/api/connection/index";
import type PingerInterface from "@/api/connection/PingerInterface";

export default class ConnectionWebSocket implements PingerInterface {
  /**
   * API host.
   * @protected
   */
  protected _host: string;
  protected _config: ConnectionSetup;
  protected socket: WebSocket | null;

  protected pingTimer: any = -1;
  protected reconnectTimer: any = -1;
  readonly pingTimeout = 5000;
  readonly name = "websocket";
  protected pingRequest: any;
  protected reConnectTimeout = 2500;
  protected counter = 0;
  public autoReconnect = false;

  protected error: Map<string, WsErrorCallback> = new Map();
  protected on: Map<string, WsConnectedCallback> = new Map();
  protected off: Map<string, WsDisconnectedCallback> = new Map();
  protected message: Map<string, WsMessageCallback> = new Map();
  protected state: Map<string, WsStateChange> = new Map();

  constructor(config: ConnectionSetup, autoConnect = false) {
    this._config = { ...config };
    let url = this._config.baseUrl;
    if (url === "/") {
      url = location.protocol + "//" + location.host;
    }

    if (url.match(/^https?:\/\//)) {
      url = url.replace(/http?/, "ws");
    }
    url = url.replace(/\/$/, "");
    this._host = `${url}/ws`;

    this.socket = null;
    if (autoConnect) {
      this.connect();
    }
  }

  get status(): boolean {
    return !!this.socket;
  }

  connect() {
    if (this.socket && this.socket.OPEN) {
      return;
    }

    clearInterval(this.reconnectTimer);
    this.autoReconnect = true;
    this.socket = new WebSocket(this._host);

    this.socket.addEventListener("error", (ev: Event) => {
      if (import.meta.env.DEV) {
        console.groupCollapsed(`[WS:event:error]`);
        console.info(ev);
        console.trace();
        console.groupEnd();
      }
      this.error.forEach((callback) => {
        callback();
      });
      if (this.autoReconnect) {
        this.reconnect();
      }
    });

    this.socket.addEventListener("open", (ev) => {
      if (import.meta.env.DEV) {
        console.groupCollapsed(`[WS:event:open]`);
        console.info(ev);
        console.trace();
        console.groupEnd();
      }
      clearInterval(this.reconnectTimer);
      this.on.forEach((callback) => {
        callback();
      });
    });

    this.socket.addEventListener("message", (ev: MessageEvent) => {
      const data = JSON.parse(ev.data);
      if (data.msg === "auth_required" || data.msg === "authentication") {
        return;
      }
      if (import.meta.env.DEV && data.msg !== "pong") {
        console.groupCollapsed(`[WS:event:message received] ${data.msg}`);
        console.info(ev);
        console.info({ ...data.msg_data });
        console.trace();
        console.groupEnd();
      }
      this.message.forEach((callback) => {
        callback(data, ev);
      });
    });

    this.socket.addEventListener("close", (ev) => {
      if (import.meta.env.DEV) {
        console.groupCollapsed(`[WS:event:close]`);
        console.info(ev);
        console.trace();
        console.groupEnd();
      }
      this.socket = null;
      this.off.forEach((callback) => {
        callback();
      });
      if (this.autoReconnect) {
        this.reconnect();
      }
    });
  }

  disconnect() {
    if (!this.socket) {
      return;
    }
    this.autoReconnect = false;
    this.cancelKeepAlive();
    this.socket.close();
    this.socket = null;
  }

  protected reconnect() {
    this.reconnectTimer = setInterval(() => {
      this.connect();
    }, this.reConnectTimeout);
  }

  doSend(data: any) {
    if (!this.socket) {
      return;
    }
    if (import.meta.env.DEV && data.msg !== "ping") {
      console.groupCollapsed(`[WS:event:message send] ${data.msg}`);
      console.info({ ...data });
      console.trace();
      console.groupEnd();
    }
    this.socket.send(
      JSON.stringify({
        ...data,
        id: ++this.counter,
      })
    );
  }

  ping() {
    if (!this.pingRequest) {
      this.pingRequest = this.sendPing()
        .then(() => {
          clearInterval(this.reconnectTimer);
          this.pingRequest = null;
          return true;
        })
        .catch((e) => {
          console.warn("WS PING FAIL", e);
          this.reconnect();
          this.pingRequest = null;
          return false;
        });
    }
    return this.pingRequest;
  }

  protected sendPing() {
    let hasResponse = false;
    return new Promise((resolve, reject) => {
      const rejectInternal = (cause: string) => {
        hasResponse = true;
        this.cancelKeepAlive();
        reject(cause);
      };
      setTimeout(() => {
        if (!hasResponse) {
          rejectInternal("noresponse");
        }
      }, this.pingTimeout);
      if (!this.socket || this.socket.readyState !== this.socket.OPEN) {
        rejectInternal("no open socket");
        return;
      }
      const time = new Date().getTime();
      this.socket.addEventListener("message", (ev) => {
        const data = JSON.parse(ev.data);
        if (data.msg === "pong" && data.msg_data.time === time) {
          hasResponse = true;
          resolve(true);
        }
      });
      this.doSend({
        kind: "req",
        msg: "ping",
        msg_data: {
          time,
        },
      });
    });
  }

  protected cancelKeepAlive() {
    if (this.pingTimer) {
      clearTimeout(this.pingTimer);
    }
  }

  addErrorCallback(name: string, callback: WsErrorCallback, replace = false) {
    if (this.error.has(name) && !replace) {
      throw `Error callback with name: "${name}" already exists`;
    }
    this.error.set(name, callback);
  }

  addConnectedCallback(
    name: string,
    callback: WsConnectedCallback,
    replace = false
  ) {
    if (this.on.has(name) && !replace) {
      throw `Connect callback with name: "${name}" already exists`;
    }
    this.on.set(name, callback);
  }

  addDisconnectedCallback(
    name: string,
    callback: WsDisconnectedCallback,
    replace = false
  ) {
    if (this.off.has(name) && !replace) {
      throw `Disconnect callback with name: "${name}" already exists`;
    }
    this.off.set(name, callback);
  }

  addMessageCallback(
    name: string,
    callback: WsMessageCallback,
    replace = false
  ) {
    if (this.message.has(name) && !replace) {
      throw `Message callback with name: "${name}" already exists`;
    }
    this.message.set(name, callback);
  }

  addStateChangeCallback(
    name: string,
    callback: WsStateChange,
    replace = false
  ) {
    if (this.state.has(name) && !replace) {
      throw `State change callback with name: "${name}" already exists`;
    }
    this.state.set(name, callback);
  }
}
