import BaseService from "@/api/services/BaseService";
import type ServiceInterface from "@/api/services/ServiceInterface";
import type {
  Activity,
  ActivityNewData,
  ActivityUpdate,
  ActivityUserInterfacePage,
  EntityCommand,
} from "@/types/activity";

export default class ServiceActivities
  extends BaseService
  implements ServiceInterface
{
  async getAll(): Promise<Activity[]> {
    const response = await this.rest.request({
      url: "/api/activities",
      method: "get",
    });
    return response.data || [];
  }

  async getActivity(activity_id: string): Promise<Activity> {
    const response = await this.rest.request({
      url: `/api/activities/${activity_id}`,
      method: "get",
    });
    return response.data as Activity;
  }

  async createNewActivity(data: ActivityNewData): Promise<Activity> {
    const response = await this.rest.request({
      url: "/api/activities",
      method: "post",
      data,
    });
    return response.data || {};
  }

  async cloneFrom(
    newActivity: ActivityNewData,
    clone_from: string
  ): Promise<Activity> {
    const data = {
      ...newActivity,
      clone_from,
    };
    return this.createNewActivity(data);
  }

  async update(activity_id: string, data: ActivityUpdate): Promise<Activity> {
    const response = await this.rest.request({
      url: `/api/activities/${activity_id}`,
      method: "patch",
      data,
    });
    return response.data || {};
  }

  async delete(data: Activity): Promise<boolean> {
    const response = await this.rest.request({
      url: `/api/activities/${data.entity_id}`,
      method: "delete",
    });
    return response.data?.code === "OK";
  }

  async addUiPage(activity: Activity, name: string) {
    const response = await this.rest.request({
      url: `/api/activities/${activity.entity_id}/ui/pages`,
      method: "post",
      data: {
        name: name,
        items: [],
      },
    });
    return response.data || {};
  }

  async deleteUiPage(activity_id: string, page_id: string): Promise<boolean> {
    const response = await this.rest.request({
      method: "delete",
      url: `/api/activities/${activity_id}/ui/pages/${page_id}`,
    });
    return response.data?.code === "OK";
  }

  async updateUiPage(activity_id: string, page: ActivityUserInterfacePage) {
    const response = await this.rest.request({
      method: "patch",
      url: `/api/activities/${activity_id}/ui/pages/${page.page_id}`,
      data: page,
    });
    return response.data;
  }

  async updateUiPageOrder(activity: Activity, page_order: string[]) {
    const response = await this.rest.request({
      url: `/api/activities/${activity.entity_id}/ui/pages`,
      method: "patch",
      data: {
        page_order: page_order,
      },
    });
    return response.data || {};
  }

  async allUiReset(activity_id: string) {
    const response = await this.rest.request({
      method: "delete",
      url: `/api/activities/${activity_id}/ui`,
    });
    return response.data;
  }

  async getPages(activity: Activity): Promise<ActivityUserInterfacePage[]> {
    const params = {};

    const response = await this.rest.request({
      url: `/api/activities/${activity.entity_id}/ui/pages`,
      method: "get",
      params,
    });
    return response.data || {};
  }

  async buttonUpdate(
    activity_id: string,
    button: string,
    command: EntityCommand
  ) {
    const response = await this.rest.request({
      method: "patch",
      url: `/api/activities/${activity_id}/buttons/${button}`,
      data: {
        button,
        short_press: command,
        // long_press: command,
      },
    });
    return response.data;
  }

  async buttonReset(activity_id: string, button: string) {
    const response = await this.rest.request({
      method: "delete",
      url: `/api/activities/${activity_id}/buttons/${button}`,
    });
    return response.data;
  }

  async allButtonsReset(activity_id: string) {
    const response = await this.rest.request({
      method: "delete",
      url: `/api/activities/${activity_id}/buttons`,
    });
    return response.data;
  }
}
