import BaseService from "@/api/services/BaseService";
import type ServiceInterface from "@/api/services/ServiceInterface";
import type {
  AvailableEntity,
  ConfiguredEntity,
  DriverConnectionTestResult,
  DriverId,
  EntityRequest,
  IntegrationDriver,
  IntegrationDriverInfo,
  IntegrationDriverUpdate,
  IntegrationInstance,
  IntegrationRequest,
  IntegrationSetupData,
  IntegrationSetupDataInput,
  IntegrationSetupInfo,
  IntegrationStatus,
  NewIntegrationData,
} from "@/types/integrationInstance";
import type { RawAxiosRequestConfig } from "axios";
import { DriverType } from "@/types/enums";

export default class ServiceIntegrations
  extends BaseService
  implements ServiceInterface
{
  async getIntegrationStatuses(): Promise<IntegrationStatus[]> {
    return this.rest.pagedGet(
      {
        url: "/api/intg",
        method: "get",
        params: {
          enabled: true,
        },
      },
      50
    );
  }

  async getInstances(): Promise<IntegrationInstance[]> {
    return this.rest.pagedGet(
      {
        url: "/api/intg/instances",
        method: "get",
        params: {
          enabled: true,
        },
      },
      50
    );
  }
  async getEnabledDrivers(): Promise<IntegrationDriverInfo[]> {
    return this.getDrivers({
      enabled: true,
    });
  }
  async getInstantiableDrivers(): Promise<IntegrationDriverInfo[]> {
    return this.getDrivers({
      instantiable: true,
      enabled: true,
    });
  }
  async getNotConfiguredDrivers(): Promise<IntegrationDriverInfo[]> {
    return this.getDrivers({
      driver_type: DriverType.EXTERNAL,
      has_instances: false,
      enabled: true,
    });
  }

  protected async getDrivers(params = {}): Promise<IntegrationDriverInfo[]> {
    return await this.rest.pagedGet(
      {
        url: "/api/intg/drivers",
        method: "get",
        params,
      },
      50
    );
  }

  async getEntities(integration_id: string, filter: string, reload: boolean = false): Promise<AvailableEntity[]> {
    const params: Record<string, any> = {};
    params.reload = reload;
    if (filter && filter.length > 0) {
      params.filter = filter;
    };
    return this.rest.pagedGet(
      {
        url: `/api/intg/instances/${integration_id}/entities`,
        method: "get",
        params: params,
      },
      50
    );
  }

  async getEntitiesByPage(integration_id: string, filter: string, reload: boolean = false, page: number = 1, searchText: string = '', entityTypes: string = ''): Promise<{data: AvailableEntity[], headers: object}> {
    const params: Record<string, any> = {};
    params.reload = reload;
    if (filter && filter.length > 0) {
      params.filter = filter;
    };
    if (searchText && searchText.length > 0) {
      params.q = searchText;
    };
    if (entityTypes && entityTypes.length > 0) {
      params.entity_types = entityTypes;
    };

    const baseGet =  this.rest.baseGet(
      {
        url: `/api/intg/instances/${integration_id}/entities`,
        method: "get",
        params: params,
      },
      50,
      page
    );
    return baseGet
  }

  async getConfiguredEntitiesByPage(integration_ids: string = '', reload: boolean = false, page: number = 1, searchText: string = '', entityTypes: string = ''): Promise<{data: AvailableEntity[], headers: object}> {
    const params: Record<string, any> = {};
    params.reload = reload;
    if (integration_ids && integration_ids.length > 0) {
      params.intg_ids = integration_ids;
    };
    if (searchText && searchText.length > 0) {
      params.q = searchText;
    };
    if (entityTypes && entityTypes.length > 0) {
      params.entity_types = entityTypes;
    };

    const baseGet =  this.rest.baseGet(
      {
        url: "/api/entities",
        method: "get",
        params: params,
      },
      50,
      page
    );
    return baseGet
  }

  async getConfiguredEntities(
    intg_id: string | null = null
  ): Promise<ConfiguredEntity[]> {
    const params: Record<string, any> = {};
    if (intg_id) {
      params.intg_id = intg_id;
    }
    return this.rest.pagedGet(
      {
        url: "/api/entities",
        method: "get",
        params,
      },
      50
    );
  }

  async addIntegration(
    driver_id: DriverId,
    data: NewIntegrationData
  ): Promise<IntegrationInstance> {
    const response = await this.rest.request({
      url: `/api/intg/drivers/${driver_id}`,
      method: "post",
      data,
    });
    return response.data;
  }

  async updateIntegration(
    inst_id: string,
    data: IntegrationRequest
  ): Promise<IntegrationInstance> {
    const response = await this.rest.request({
      url: `/api/intg/instances/${inst_id}`,
      method: "patch",
      data,
    });
    return response.data;
  }

  async getIntegration(inst_id: string): Promise<IntegrationInstance> {
    const response = await this.rest.request({
      method: "get",
      url: `/api/intg/instances/${inst_id}`,
    });
    return response.data;
  }

  async getDriver(driver_id: string): Promise<IntegrationDriver> {
    const response = await this.rest.request({
      method: "get",
      url: `/api/intg/drivers/${driver_id}`,
    });
    return response.data;
  }

  async addInstanceEntity(
    inst: IntegrationInstance,
    entity: AvailableEntity
  ): Promise<ConfiguredEntity> {
    const response = await this.rest.request({
      method: "post",
      url: `/api/intg/instances/${inst.integration_id}/entities/${entity.entity_id}`,
      data: {
        name: entity.name,
      },
    });
    return response.data;
  }

  async addInstanceEntities(
    inst: IntegrationInstance,
    entity_ids: string[]
  ): Promise<string[]> {
    const data = entity_ids;
    const response = await this.rest.request({
      method: "post",
      url: `/api/intg/instances/${inst.integration_id}/entities`,
      data: data
    });
    return response.data;
  }

  async updateEntity(
    entity_id: string,
    changes: EntityRequest
  ): Promise<ConfiguredEntity> {
    const response = await this.rest.request({
      method: "patch",
      url: `/api/entities/${entity_id}`,
      data: changes,
    });
    return response.data;
  }

  async removeEntity(entity: ConfiguredEntity) {
    const response = await this.rest.request({
      method: "delete",
      url: `/api/entities/${entity.entity_id}`,
    });
    return response.data;
  }

  async removeEntities(entity_ids: string[]) {
    const data = {
      entity_ids: entity_ids
    }
    const response = await this.rest.request({
      method: "delete",
      url: `/api/entities`,
      data: data
    });
    return response.data;
  }

  async connectInst(inst_id: string): Promise<IntegrationInstance> {
    return this.changeConnectionStatus(inst_id, true);
  }
  async disconnectInst(inst_id: string): Promise<IntegrationInstance> {
    return this.changeConnectionStatus(inst_id, false);
  }
  protected async changeConnectionStatus(
    inst_id: string,
    connect: boolean
  ): Promise<IntegrationInstance> {
    await this.rest.request({
      url: `/api/intg/instances/${inst_id}`,
      method: "put",
      params: {
        cmd: connect ? "CONNECT" : "DISCONNECT",
      },
    });
    return this.getIntegration(inst_id);
  }

  async deleteIntegration(integration: IntegrationStatus): Promise<boolean> {
    const config: RawAxiosRequestConfig = {
      method: "delete",
    };
    if (integration.integration_id) {
      config.url = `/api/intg/instances/${integration.integration_id}`;
    } else if (integration.driver_id && !integration.integration_id) {
      config.url = `/api/intg/drivers/${integration.driver_id}`;
    } else {
      throw "Cannot delete integration";
    }
    const response = await this.rest.request(config);
    return response.data.code === "OK";
  }

  async updateDriver(
    driver_id: string,
    data: IntegrationDriverUpdate
  ): Promise<IntegrationDriver> {
    const response = await this.rest.request({
      method: "patch",
      url: `/api/intg/drivers/${driver_id}`,
      data,
    });
    return response.data;
  }

  async startDiscovery() {
    const response = await this.rest.request({
      method: "put",
      url: "/api/intg/discover",
    });
    return response.data.code === "OK";
  }
  async stopDiscovery() {
    const response = await this.rest.request({
      method: "delete",
      url: "/api/intg/discover",
    });
    return response.data.code === "OK";
  }

  async testConnection(
    driver_id: string,
    driver_url: string,
    token: string | undefined = undefined
  ): Promise<DriverConnectionTestResult> {
    const request = await this.rest.request({
      method: "put",
      url: `/api/intg/discover/${driver_id}`,
      params: {
        cmd: "CONNECTION_TEST",
      },
      data: {
        connection: {
          driver_url,
          token,
        },
      },
    });
    const result: DriverConnectionTestResult = {
      code: 200,
      result: false,
      message: request.data.message,
    };
    if (request.data.driver_id === driver_id) {
      result.result = true;
      result.driver = request.data;
    }
    return result;
  }

  async registerIntegration(
    driver_id: string,
    driver_url: string,
    token: string | undefined = undefined
  ): Promise<IntegrationDriver> {
    const request = await this.rest.request({
      method: "post",
      url: `/api/intg/discover/${driver_id}`,
      data: {
        driver_url,
        token,
      },
    });
    return await this.getDriver(request.data.driver_id);
  }

  async startSetupIntegration(
    driver_id: string,
    data: IntegrationSetupDataInput
  ): Promise<IntegrationSetupInfo> {
    const response = await this.rest.request({
      method: "post",
      url: "/api/intg/setup",
      data: {
        driver_id,
        setup_data: data.input_values || {},
      },
    });
    return response.data;
  }

  async continueSetupIntegration(
    driver_id: string,
    data: IntegrationSetupData
  ): Promise<IntegrationSetupInfo> {
    const response = await this.rest.request({
      method: "put",
      url: `/api/intg/setup/${driver_id}`,
      data,
    });
    return response.data;
  }

  async stopSetupIntegration(driver_id: string): Promise<boolean> {
    const response = await this.rest.request({
      method: "delete",
      url: `/api/intg/setup/${driver_id}`,
    });
    return response.data.code === "OK";
  }
}
