import BaseService from "@/api/services/BaseService";

import type ServiceInterface from "@/api/services/ServiceInterface";
import type { Remote, RemoteNewData, RemoteUpdate } from "@/types/remote";
import type { IrCodeDefinition, RemoteDataSet, RemoteIrCode } from "@/types/ir";
import type { ActivityUserInterfacePage } from "@/types/activity";

export default class ServiceRemotes
  extends BaseService
  implements ServiceInterface
{
  async getAll(): Promise<Remote[]> {
    const response = await this.rest.request({
      url: "/api/remotes?page=1&limit=100",
      method: "get",
    });
    return response.data || [];
  }

  async createNewRemote(data: RemoteNewData): Promise<Remote> {
    const response = await this.rest.request({
      url: "/api/remotes",
      method: "post",
      data,
    });
    return response.data || {};
  }

  async update(entity_id: string, data: RemoteUpdate): Promise<Remote> {
    const response = await this.rest.request({
      url: `/api/remotes/${entity_id}`,
      method: "patch",
      data,
    });
    return response.data || {};
  }

  async delete(remote: Remote): Promise<boolean> {
    const response = await this.rest.request({
      url: `/api/remotes/${remote.entity_id}`,
      method: "delete",
    });
    return response.data?.code === "OK";
  }

  async getRemote(entity_id: string): Promise<Remote> {
    const response = await this.rest.request({
      url: `/api/remotes/${entity_id}`,
      method: "get",
    });
    return response.data || {};
  }

  async getRemoteIrCodes(entity_id: string): Promise<RemoteDataSet> {
    const response = await this.rest.request({
      method: "get",
      url: `/api/remotes/${entity_id}/ir`,
    });
    return response.data;
  }

  async addCustomCodeToSet(
    entity_id: string,
    cmd_id: string,
    code: IrCodeDefinition
  ): Promise<RemoteIrCode> {
    const response = await this.rest.request({
      method: "post",
      url: `/api/remotes/${entity_id}/ir/${cmd_id}`,
      data: code,
    });
    return response.data;
  }

  async modifyCustomCodeToSet(
    entity_id: string,
    cmd_id: string,
    code: IrCodeDefinition
  ): Promise<RemoteIrCode> {
    const response = await this.rest.request({
      method: "patch",
      url: `/api/remotes/${entity_id}/ir/${cmd_id}`,
      data: code,
    });
    return response.data;
  }

  async removeCustomCodeFromSet(
    entity_id: string,
    cmd_id: string
  ): Promise<boolean> {
    const response = await this.rest.request({
      url: `/api/remotes/${entity_id}/ir/${cmd_id}`,
      method: "delete",
    });
    return response.data?.code === "OK";
  }

  async addUiPage(remote: Remote, name: string) {
    const response = await this.rest.request({
      url: `/api/remotes/${remote.entity_id}/ui/pages`,
      method: "post",
      data: {
        name: name,
        items: [],
      },
    });
    return response.data || {};
  }

  async updateUiPageOrder(remote: Remote, page_order: string[]) {
    const response = await this.rest.request({
      url: `/api/remotes/${remote.entity_id}/ui/pages`,
      method: "patch",
      data: {
        page_order: page_order,
      },
    });
    return response.data || {};
  }

  async getPages(remote: Remote): Promise<ActivityUserInterfacePage[]> {
    const params = {};

    const response = await this.rest.request({
      url: `/api/remotes/${remote.entity_id}/ui/pages`,
      method: "get",
      params,
    });
    return response.data || {};
  }

  async deleteUiPage(remote_id: string, page_id: string): Promise<boolean> {
    const response = await this.rest.request({
      method: "delete",
      url: `/api/remotes/${remote_id}/ui/pages/${page_id}`,
    });
    return response.data?.code === "OK";
  }

  async updateUiPage(remote_id: string, page: ActivityUserInterfacePage) {
    const response = await this.rest.request({
      method: "patch",
      url: `/api/remotes/${remote_id}/ui/pages/${page.page_id}`,
      data: page,
    });
    return response.data;
  }

  async allUiReset(remote_id: string) {
    const response = await this.rest.request({
      method: "delete",
      url: `/api/remotes/${remote_id}/ui`,
    });
    return response.data;
  }

  async buttonUpdate(remote_id: string, button: string, command: string) {
    const response = await this.rest.request({
      method: "patch",
      url: `/api/remotes/${remote_id}/buttons/${button}`,
      data: {
        button,
        short_press: { cmd_id: command },
        // long_press: { cmd_id: command },
      },
    });
    return response.data;
  }

  async buttonReset(remote_id: string, button: string) {
    const response = await this.rest.request({
      method: "delete",
      url: `/api/remotes/${remote_id}/buttons/${button}`,
    });
    return response.data;
  }

  async allButtonsReset(remote_id: string) {
    const response = await this.rest.request({
      method: "delete",
      url: `/api/remotes/${remote_id}/buttons`,
    });
    return response.data;
  }
}
