import type {
  Activity,
  CommandItem,
  CommandParameter,
  CommandSequenceListItem,
  EntityCmdParamBool,
  EntityCmdParamEnum,
  EntityCmdParamNumber,
  EntityCmdParamRegex,
  EntityCommandMetadata,
  IncludedEntity,
} from "@/types/activity";
import type {
  CommandSequence,
  CommandSequenceDelay,
  CommandSequenceEntity,
} from "@/types/command";
import type { ConfiguredEntity } from "@/types/integrationInstance";
import { SequenceType } from "@/types/enums";

export function getAvailableCommandsForActivity(
  activity: Activity,
  commandMetadata: EntityCommandMetadata[]
): CommandItem[] {
  const commands: CommandItem[] = [];
  (activity.options?.included_entities || []).forEach(
    (entity: IncludedEntity) => {
      commands.push(...getAvailableEntityCommands(entity, commandMetadata));
    }
  );
  return commands;
}

export function getAvailableEntityCommands(
  entity: IncludedEntity,
  commandMetadata: EntityCommandMetadata[]
): CommandItem[] {
  const commands: string[] = [];

  if (entity.entity_commands && entity.entity_commands.length) {
    commands.push(...entity.entity_commands);
  }
  if (entity.simple_commands && entity.simple_commands.length) {
    commands.push(...entity.simple_commands);
  }
  return commands.map((id) => {
    let type: "simple" | "entity" | null = null;
    if (entity.simple_commands?.includes(id)) {
      type = "simple";
    } else if (entity.entity_commands?.includes(id)) {
      type = "entity";
    }

    let cmd;
    if (type === "entity") {
      cmd = commandMetadata.find((cmd: EntityCommandMetadata) => {
        return cmd.id === id;
      });
    } else if (type === "simple") {
      cmd = {
        id,
        cmd_id: id,
        name: { en: id },
        entity_id: entity.entity_id,
        entity_name: entity.name,
      } as EntityCommandMetadata;
    }
    return {
      command_type: type,
      type: SequenceType.command,
      id: entity.entity_id + ":" + id,
      cmd: cmd as EntityCommandMetadata,
      entity,
    };
  });
}

export function sequencesToListItems(
  sequences: CommandSequence[],
  allEntities: ConfiguredEntity[],
  commandMetadata: EntityCommandMetadata[]
): CommandSequenceListItem[] {
  return sequences.map((sequence: CommandSequence, pos: number) => {
    if (sequence.type === SequenceType.delay) {
      return {
        command_type: "delay",
        pos,
        type: SequenceType.delay,
        sequence,
      };
    }
    const entity = allEntities.find((item: IncludedEntity) => {
      return item.entity_id === sequence.command.entity_id;
    });
    let cmd = commandMetadata.find((item: EntityCommandMetadata) => {
      return item.id === sequence.command.cmd_id;
    });

    // Simple entity command
    if (
      !cmd &&
      sequence.command.cmd_id &&
      entity?.options?.simple_commands?.length &&
      entity?.options?.simple_commands.includes(sequence.command.cmd_id)
    ) {
      cmd = {
        id: sequence.command.cmd_id,
        cmd_id: sequence.command.cmd_id,
        name: { en: sequence.command.cmd_id },
        entity_name: entity.name,
        entity_id: entity.entity_id,
      };
    }
    return {
      command_type: "entity",
      pos,
      id: entity?.entity_id + ":" + cmd?.id,
      type: SequenceType.command,
      cmd,
      entity,
      sequence,
    };
  });
}

export function createSequenceForUpdate(
  items: CommandSequenceListItem[],
  entity_ids: string[]
): CommandSequence[] {
  return items
    .map((item) => {
      if (item.type === SequenceType.delay) {
        return (
          item.sequence ||
          ({
            type: "delay",
            delay: 0,
          } as CommandSequenceDelay)
        );
      }

      const sequence = item.sequence as CommandSequenceEntity;

      // handle new step in sequence
      if (!sequence && item.cmd) {
        return createNewSequenceItem(item);
      }

      // update existing step in sequence
      return sequence;
    })
    .filter((item) => {
      if (item.type === SequenceType.delay || !item.command?.entity_id) {
        return true;
      }
      return entity_ids.includes(item.command.entity_id);
    });
}

function createNewSequenceItem(
  item: CommandSequenceListItem
): CommandSequenceEntity {
  if (!item.cmd) {
    throw `item.cmd must be set for a new command item`;
  }

  if (item.command_type === "simple") {
    return {
      type: "command",
      command: {
        cmd_id: item.cmd.cmd_id,
        entity_id: item.cmd.entity_id,
      },
    } as CommandSequenceEntity;
  } else if (item.command_type === "entity") {
    const params = createNewCommandParams(item.cmd);
    return {
      type: item.type,
      command: {
        cmd_id: item.cmd.id,
        entity_id: item.entity?.entity_id,
        params: params,
      },
    } as CommandSequenceEntity;
  } else {
    throw `only simple and entity command_type allowed`;
  }
}

function createNewCommandParams(
  entityCommandMetadata: EntityCommandMetadata
): CommandParameter {
  let params = {};
  if (Array.isArray(entityCommandMetadata.params)) {
    for (const param of entityCommandMetadata.params) {
      if (param.type === "number") {
        params = {
          ...params,
          ...createEntityCmdParamNumber(param as EntityCmdParamNumber),
        };
      } else if (param.type === "bool") {
        params = {
          ...params,
          ...createEntityCmdParamBool(param as EntityCmdParamBool),
        };
      } else if (param.type === "regex") {
        params = {
          ...params,
          ...createEntityCmdParamRegex(param as EntityCmdParamRegex),
        };
      } else if (param.type === "enum") {
        params = {
          ...params,
          ...createEntityCmdParamEnum(param as EntityCmdParamEnum),
        };
      } else {
        console.warn(`Unsupported parameter type: ${param.name}`);
      }
    }
  }

  return params;
}

function createEntityCmdParamNumber(
  param: EntityCmdParamNumber
): CommandParameter {
  // TODO default value. Check if defined in API, if not: decide if we should add it.
  return {
    [param.param]: param.min || 0,
  } as CommandParameter;
}

function createEntityCmdParamBool(param: EntityCmdParamBool) {
  // TODO default value. Check if defined in API, if not: decide if we should add it.
  return {
    [param.param]: false,
  } as CommandParameter;
}

function createEntityCmdParamRegex(param: EntityCmdParamRegex) {
  // TODO default value. Check if defined in API, if not: decide if we should add it.
  // TODO check if for a regex value the param object could be omitted for the initial save, because we most likely don't have a valid value
  return {
    // [param.param]: "",
  } as CommandParameter;
}

function createEntityCmdParamEnum(param: EntityCmdParamEnum) {
  // TODO default value. Check if defined in API, if not: decide if we should add it.
  if (param.values.length > 0) {
    return {
      [param.param]: param.values[0],
    } as CommandParameter;
  } else {
    return {} as CommandParameter;
  }
}
