import { onMounted, onUnmounted, ref, reactive } from "vue";

import type { ComponentPublicInstance } from "vue";
import type {
  ActivityUserInterfaceGridItem,
  ActivityUserInterfaceItem,
  ActivityUserInterfacePage,
  DeviceButtonMapping,
} from "@/types/activity";
import type { GridLayoutComponent, GridItemComponent } from "@/types/grid";

/**
 * Given two layoutitems, check if they collide.
 *
 * @return {Boolean}   True if colliding.
 */
export function collides(
  l1: ActivityUserInterfaceGridItem,
  l2: ActivityUserInterfaceGridItem
): boolean {
  // same element.
  if (l1.item === l2.item) {
    return false;
  }

  // l1 is left of l2
  if (l1.x + l1.w <= l2.x) {
    return false;
  }

  // l1 is right of l2
  if (l1.x >= l2.x + l2.w) {
    return false;
  }

  // l1 is above l2
  if (l1.y + l1.h <= l2.y) {
    return false;
  }

  // l1 is below l2
  if (l1.y >= l2.y + l2.h) {
    return false;
  }
  // boxes overlap
  return true;
}

export function getAllCollisions(
  layout: ActivityUserInterfaceGridItem[],
  layoutItem: ActivityUserInterfaceGridItem
): ActivityUserInterfaceGridItem[] {
  return layout.filter((l) => {
    return collides(l, layoutItem);
  });
}

export function validateChange(
  items: ActivityUserInterfaceItem[],
  updatedIndex: number,
  newValues: ActivityUserInterfaceItem
) {
  const list = items.map((item, index) => {
    return {
      x: item.location?.x || 0,
      y: item.location?.y || 0,
      w: item.size?.width || 1,
      h: item.size?.height || 1,
      i: index.toString(),
      item,
    } as ActivityUserInterfaceGridItem;
  });
  const checkItem = {
    ...list[updatedIndex],
    w: newValues.size.width,
    h: newValues.size.height,
  };
  return getAllCollisions(list, checkItem).length === 0;
}

export function useEditorKeyboardEvents(
  addUiPage: () => Promise<void>,
  savePage: (pageIndex: number) => Promise<void>
) {
  const dragAreas = ref<Record<string, GridLayoutComponent>>({});
  const gridItems = ref<Record<string, GridItemComponent>>({});
  const layouts = ref<ActivityUserInterfaceGridItem[][]>([]);

  const pages = ref<ActivityUserInterfacePage[]>([]);
  const editButton = ref<ActivityUserInterfaceItem | null>(null);
  const editButtonCoord = ref<{ page: number; index: number } | null>(null);
  const editPopup = ref();
  const activePage = ref(0);
  const editPhysicalButton = ref<Object | null>(null);
  const editPhysicalButtonPopup = ref();

  const physicalButtonPopupPosition = ref<DOMRect>()

  function getPageIndexById(pageId: string): number {
    return pages.value.findIndex((item) => {
      return item.page_id === pageId;
    });
  }

  function registerDragArea(
    pageId: string,
    el: Element | ComponentPublicInstance
  ) {
    dragAreas.value[pageId] = el as GridLayoutComponent;
  }

  function registerGridItem(
    pageId: string,
    index: number,
    el: Element | ComponentPublicInstance
  ) {
    gridItems.value[pageId + ":" + index] = el as GridItemComponent;
  }

  function closeButtonEdit() {
    editButton.value = null;
    editButtonCoord.value = null;
    setTimeout(() => {
      editPhysicalButton.value = null;
    }, 50);
  }
  function startButtonEdit(
    page: number,
    index: number,
    element: ActivityUserInterfaceItem,
    $event: MouseEvent
  ) {
    closeButtonEdit();
    setTimeout(() => {
      editButton.value = element;
      editButtonCoord.value = {
        page,
        index,
      };
      const item: HTMLDivElement = ($event.target as HTMLDivElement).parentNode as HTMLDivElement;
      const pos = item.getBoundingClientRect();
      const triangle = editPopup.value.querySelector('.triangle');
      editPopup.value.style.left = pos.left - pos.width - 115  + "px";

      // Calculate the distance of the popup from the bottom of the screen - constant is popup max height
      setTimeout(() => {
        let popupHeight = editPopup.value.getBoundingClientRect().height + 10 || 620;
        const distanceFromBottom = window.innerHeight - (pos.top + popupHeight);

        if (distanceFromBottom < 0) {
          editPopup.value.style.top = pos.top + distanceFromBottom + "px";
          triangle.style.marginTop = -distanceFromBottom + "px";
        } else {
          editPopup.value.style.top = pos.top + "px";
          triangle.style.marginTop = 0;
        }
      }, 10);
    }, 100);
  }

  function startPhysicalButtonEdit(
    element: DeviceButtonMapping,
    $event: MouseEvent
  ) {
    closeButtonEdit();

    setTimeout(() => {
      editPhysicalButton.value = element;

      const item: HTMLDivElement = ($event.target as HTMLDivElement)
        .parentNode as HTMLDivElement;
      physicalButtonPopupPosition.value = item.getBoundingClientRect();

      setPopupPosition(physicalButtonPopupPosition.value);
    }, 100);
  }

  function updateGridItem(newValues: ActivityUserInterfaceItem) {
    const { page, index } = editButtonCoord.value as {
      page: number;
      index: number;
    };

    const item = {
      ...pages.value[page].items[index],
      type: newValues.type,
      size: newValues.size,
      command: newValues.command,
    };
    if (newValues.type === "text") {
      item.text = newValues.text;
    }
    if (newValues.type === "icon") {
      item.icon = newValues.icon;
    }

    pages.value[page].items[index] = item;
    editButton.value = item;
    savePage(page);
  }

  function updatePhysicalButtonEdit(
    element: DeviceButtonMapping,
  ) {
    
    editPhysicalButton.value = element;

    if (physicalButtonPopupPosition.value) {
      setPopupPosition(physicalButtonPopupPosition.value);
    }
  }

  function deleteGridItem() {
    if (!editButton.value) {
      return;
    }
    const { page, index } = editButtonCoord.value as {
      page: number;
      index: number;
    };
    pages.value[page].items.splice(index, 1);
    closeButtonEdit();
    savePage(page);
  }

  function movedGridItem(
    pageId: string,
    index: number,
    item: ActivityUserInterfaceGridItem,
    changes: { i: string; newX: number; newY: number }
  ) {
    const pageIndex = getPageIndexById(pageId);
    pages.value[pageIndex].items[index] = {
      ...pages.value[pageIndex].items[index],
      location: {
        x: changes.newX,
        y: changes.newY,
      },
    };
    savePage(pageIndex);
  }

  function buildLayout(items: ActivityUserInterfaceItem[]) {
    return items.map((item, index: number) => {
      return {
        x: item.type === "numpad" ? 0 : item.location.x,
        y: item.type === "numpad" ? 0 : item.location.y,
        w: item.size?.width || 1,
        h: item.size?.height || 1,
        i: index.toString(),
        item,
      };
    });
  }

  function onKeyup(event: KeyboardEvent) {
    if (event.code === "Escape") {
      closeButtonEdit();
    }
  }

  function onClick(event: MouseEvent) {
    if (!editButton.value && !editPhysicalButton.value) {
      return;
    }

    const path = (event.composedPath() as HTMLElement[]) || [];

    const editIcon = path.find((el) => {
      if (!el || !el.classList) {
        return false;
      }
      return el.classList.contains("ui-page--item--edit");
    });

    const popupTarget = path.find((el) => {
      return el === editPopup.value || el === editPhysicalButtonPopup.value;
    });

    const modalCommand = path.find((el) => {
      if (!el || !el.classList) {
        return false;
      }
      return el.classList.contains("modal-mask");
    });

    if (!editIcon && !popupTarget && !modalCommand) {
      closeButtonEdit();
    }
  }

  const mouseXY = { x: 0, y: 0 };
  const DragPos = { x: 0, y: 0, w: 1, h: 1, i: "drop" };
  let creatingPage = false;
  async function changeToNextEmptyPage() {
    let next = null;
    for (let i = activePage.value; i < pages.value.length; i++) {
      if (!pages.value[i].items.length) {
        next = i;
      }
    }
    if (next !== null) {
      activePage.value = next;
      return;
    }
    if (creatingPage) {
      return;
    }
    creatingPage = true;
    await addUiPage();
    creatingPage = false;
  }

  async function prototypeDrag(
    e: DragEvent,
    element: ActivityUserInterfaceItem
  ) {
    if (
      element.type === "numpad" &&
      layouts.value[activePage.value].length > 0
    ) {
      await changeToNextEmptyPage();
      return;
    }

    const pageId = pages.value[activePage.value].page_id;
    const dragArea = dragAreas.value[pageId];
    const parentRect = dragArea.$el.getBoundingClientRect();
    const mouseInGrid =
      mouseXY.x > parentRect.left &&
      mouseXY.x < parentRect.right &&
      mouseXY.y > parentRect.top &&
      mouseXY.y < parentRect.bottom;
    const layout = layouts.value[activePage.value];
    const index = layout.findIndex((item) => {
      return item.i === "drop";
    });
    if (mouseInGrid && index === -1) {
      const gridItem = {
        x: (layouts.value[activePage.value].length * 2) % 4,
        y: layouts.value[activePage.value].length + 4,
        w: 1,
        h: 1,
        i: "drop",
        item: element,
      };
      if (element.type === "numpad") {
        gridItem.x = 0;
        gridItem.y = 0;
      }
      layouts.value[activePage.value].push(gridItem);
      try {
        gridItems.value[pageId + ":" + (layout.length - 1)].$el.style.display =
          "none";
      } catch {}
    }
    if (index === -1) {
      return;
    }

    const key = pageId + ":" + index;
    try {
      gridItems.value[key].$el.style.display = "none";
    } catch (er) {
      // console.error(er);
    }

    const el = gridItems.value[key];
    el.$el.dragging = {
      top: mouseXY.y - parentRect.top,
      left: mouseXY.x - parentRect.left,
    };
    const new_pos = el.calcXY(
      mouseXY.y - parentRect.top,
      mouseXY.x - parentRect.left
    );
    if (mouseInGrid) {
      dragAreas.value[pageId].dragEvent(
        "dragstart",
        "drop",
        new_pos.x,
        new_pos.y,
        1,
        1
      );
      DragPos.i = String(index);
      if (element.type === "numpad") {
        DragPos.x = 0;
        DragPos.y = 0;
      } else {
        DragPos.x = layout[index].x;
        DragPos.y = layout[index].y;
      }
    } else {
      dragAreas.value[pageId].dragEvent(
        "dragend",
        "drop",
        new_pos.x,
        new_pos.y,
        1,
        1
      );
      layouts.value[activePage.value] = layouts.value[activePage.value].filter(
        (obj) => {
          return obj.i !== "drop";
        }
      );
    }
  }

  async function prototypeDragEnd(
    e: DragEvent,
    element: ActivityUserInterfaceItem
  ) {
    const pageId = pages.value[activePage.value].page_id;
    const dragArea = dragAreas.value[pageId];
    const parentRect = dragArea.$el.getBoundingClientRect();
    const mouseInGrid =
      mouseXY.x > parentRect.left &&
      mouseXY.x < parentRect.right &&
      mouseXY.y > parentRect.top &&
      mouseXY.y < parentRect.bottom;
    if (mouseInGrid) {
      console.info(
        `Dropped element props:\n${JSON.stringify(
          DragPos,
          ["x", "y", "w", "h"],
          2
        )}`
      );
      pages.value[activePage.value].items.push({
        ...element,
        location: {
          x: DragPos.x,
          y: DragPos.y,
        },
      });
      dragAreas.value[pageId].dragEvent(
        "dragend",
        "drop",
        DragPos.x,
        DragPos.y,
        1,
        1
      );
      layouts.value[activePage.value] = layouts.value[activePage.value].filter(
        (obj) => {
          return obj.i !== "drop";
        }
      );
      layouts.value[activePage.value].push({
        x: DragPos.x,
        y: DragPos.y,
        w: 1,
        h: 1,
        i: DragPos.i,
        item: element,
      });
      dragAreas.value[pageId].dragEvent(
        "dragend",
        DragPos.i,
        DragPos.x,
        DragPos.y,
        1,
        1
      );
      try {
        Object.keys(gridItems.value).forEach((key) => {
          gridItems.value[key].$el.style.display = "block";
        });
      } catch (err) {
        console.error(err);
      }

      await savePage(activePage.value);

      setTimeout(() => {
        const gridItemSize = 58;
        openEditPopupByClick(parentRect.x + DragPos.x * gridItemSize + 5, parentRect.y + DragPos.y * gridItemSize + 5);
      }, 50);
    }
  }

  function openEditPopupByClick(x: number, y: number) {
    const event = new MouseEvent('click', {
      clientX: x,
      clientY: y,
      bubbles: true,
      cancelable: true,
      view: window
    });
    
    const targetElementParent = document.elementFromPoint(x, y)?.parentElement?.parentElement;

    if (targetElementParent) {
      const targetElement = targetElementParent.querySelector('.icon-edit');

      if (targetElement) {
        targetElementParent.classList.add('start-edit')
        targetElement.dispatchEvent(event);
        targetElementParent.classList.remove('start-edit')
      }
    }
  }

  async function prototypeAdd(
    ev: PointerEvent | MouseEvent,
    element: ActivityUserInterfaceItem
  ) {
    const gridItem = {
      x: 0,
      y: 0,
      w: 1,
      h: 1,
      i: "drop",
      item: element,
    };
    findLoop: for (let y = 0; y < 6; y++) {
      for (let x = 0; x < 4; x++) {
        const item = {
          ...gridItem,
          x,
          y,
        };
        const collides = getAllCollisions(
          layouts.value[activePage.value],
          item
        );
        if (!collides.length) {
          gridItem.x = x;
          gridItem.y = y;
          break findLoop;
        }
      }
    }

    pages.value[activePage.value].items.push({
      ...element,
      location: {
        x: gridItem.x,
        y: gridItem.y,
      },
    });
    layouts.value[activePage.value].push(gridItem);

    await savePage(activePage.value);
  }

  function setPopupPosition(itemPosition: DOMRect) {
    const offsetTop = 100;
    const pos = itemPosition;

    const triangle = editPhysicalButtonPopup.value.querySelector('.triangle');
    editPhysicalButtonPopup.value.style.left = pos.left + 280 + "px";

    // Calculate the distance of the popup from the bottom of the screen - constant is popup max height
    setTimeout(() => {
      let popupHeight = editPhysicalButtonPopup.value.getBoundingClientRect().height + 10 || 500;

      const distanceFromBottom = window.innerHeight - (pos.top + popupHeight);

      if (-distanceFromBottom + 50 > popupHeight) {
        editPhysicalButtonPopup.value.style.top = pos.top + distanceFromBottom + 2 + "px";
        triangle.style.marginTop = -distanceFromBottom - 15 + "px";
      } else if (distanceFromBottom < 0) {
        editPhysicalButtonPopup.value.style.top = pos.top + distanceFromBottom - 30 + "px";
        triangle.style.marginTop = -distanceFromBottom + 20 + "px";
      } else {
        editPhysicalButtonPopup.value.style.top = pos.top  - offsetTop + "px";
        triangle.style.marginTop = offsetTop - 10 + "px";
      }
    }, 10);

  }

  function onDragOver(e: DragEvent) {
    mouseXY.x = e.clientX;
    mouseXY.y = e.clientY;
  }

  onMounted(() => {
    document.addEventListener("keyup", onKeyup);
    document.addEventListener("click", onClick);
    document.addEventListener("dragover", onDragOver, false);
  });
  onUnmounted(() => {
    document.removeEventListener("keyup", onKeyup);
    document.removeEventListener("click", onClick);
    document.removeEventListener("dragover", onDragOver, false);
  });

  return {
    dragAreas,
    gridItems,
    registerDragArea,
    registerGridItem,

    pages,
    activePage,
    layouts,
    editButton,
    editButtonCoord,
    editPopup,
    editPhysicalButton,
    editPhysicalButtonPopup,
    closeButtonEdit,
    startPhysicalButtonEdit,
    startButtonEdit,
    movedGridItem,
    updateGridItem,
    updatePhysicalButtonEdit,
    deleteGridItem,
    buildLayout,

    getPageIndexById,
    prototypeDrag,
    prototypeDragEnd,
    prototypeAdd,
  };
}
