import type { LanguageText } from "@/types/config";
import i18n from "@/i18n";

export function getCurrentLocale() {
  return i18n.global.locale.value;
}
export function getCurrentLangcode() {
  return getCurrentLocale().replace(/_.*$/, "");
}

/**
 * Retrieve language specific text of the current locale.
 *
 * If there's no locale specific text (e.g. "de_CH"), the language specific text is searched (e.g. "de").
 * If still not found, english ("en") is searched, with the first language in the `text` object as fallback.
 *
 * @param {LanguageText | undefined} text The language text map, key is the language identifier, value the language specific text.
 */
export default function translatedProperty(
  text: LanguageText | undefined
): string {
  if (!text) {
    return "";
  }

  return (
    text[getCurrentLocale()] ||
    // then just the language code, e.g. "de"
    text[getCurrentLangcode()] ||
    // use default language: "en"
    text["en"] ||
    // fallback: use first language text
    text[Object.keys(text)[0]] ||
    // safety net for empty object
    ""
  );
}

/**
 * Perform a case-insensitive text search in the provided language text map.
 *
 * @param {LanguageText | undefined} text The language text map, key is the language identifier, value the language specific text.
 * @param {string} search The text to search within the language text.
 * @param {string} defaultText The text to return if `text` is empty or the search didn't find a match.
 */
export function searchLanguageText(
  text: LanguageText | undefined,
  search: string,
  defaultText = ""
) {
  if (!text) {
    return defaultText;
  }

  for (const value of Object.values(text)) {
    if (value.toLowerCase().includes(search)) {
      return value;
    }
  }

  return defaultText;
}
