import { defineStore } from "pinia";
import ApiConnection from "@/api";
import type {
  Activity,
  ActivityNewData,
  ActivityUpdate,
  ActivityUserInterfacePage,
  EntityCommand,
} from "@/types/activity";

const API = ApiConnection.activities;

export const activitiesStore = defineStore({
  id: "activities",
  state: () => ({
    inited: false,
    activities: [] as Activity[] | [],
    pages: [] as ActivityUserInterfacePage[] | [],
  }),

  actions: {
    init() {
      if (this.$state.inited) {
        return;
      }
      this.$state.inited = true;
      ApiConnection.websocket().addMessageCallback("activities", (data) => {
        if (
          data.msg === "entity_change" &&
          data.msg_data?.new_state?.entity_type === "activity"
        ) {
          this.getAll();
        }
        this.socketUpdate(data.msg_data || {});
      });
    },

    async socketUpdate(msg_data: Record<string, any>) {
      // This is only a placeholder action to make possible to subscribe on.
    },

    async getAll(): Promise<Activity[]> {
      this.init();
      try {
        const items = await API.getAll();
        this.$state.activities = items;
        return items;
      } catch (e) {
        return [];
      }
    },

    async getActivity(activity_id: string): Promise<Activity> {
      this.init();
      return await API.getActivity(activity_id);
    },

    async create(activityData: ActivityNewData): Promise<Activity> {
      return await API.createNewActivity(activityData);
    },

    async clone(
      newActivity: ActivityNewData,
      clone_from: string
    ): Promise<Activity> {
      return await API.cloneFrom(newActivity, clone_from);
    },

    async update(
      activity_id: string,
      activity: ActivityUpdate
    ): Promise<Activity> {
      return await API.update(activity_id, activity);
    },

    async delete(activity: Activity): Promise<Activity[]> {
      await API.delete(activity);
      return this.getAll();
    },

    async addUiPage(activity: Activity, name: string): Promise<Activity> {
      await API.addUiPage(activity, name);
      return this.getActivity(activity.entity_id);
    },

    async deleteUiPage(
      activity_id: string,
      page_id: string
    ): Promise<Activity> {
      await API.deleteUiPage(activity_id, page_id);
      return this.getActivity(activity_id);
    },

    async updateUiPage(
      activity_id: string,
      page: ActivityUserInterfacePage
    ): Promise<Activity> {
      await API.updateUiPage(activity_id, page);
      return this.getActivity(activity_id);
    },

    async allUiReset(activity_id: string): Promise<Activity> {
      await API.allUiReset(activity_id);
      return this.getActivity(activity_id);
    },

    async updatePagesOrder(
      activity: Activity,
      pageValues: ActivityUserInterfacePage[]
    ) {
      const pages: string[] = Array.from(pageValues).map((page) => {
        return page.page_id;
      });
      await API.updateUiPageOrder(activity, pages);

      return this.getPages(activity);
    },

    async getPages(activity: Activity) {
      this.$state.pages = await API.getPages(activity);
      return this.$state.pages;
    },

    async buttonUpdate(
      activity_id: string,
      button: string,
      command: EntityCommand
    ): Promise<Activity> {
      await API.buttonUpdate(activity_id, button, command);
      return this.getActivity(activity_id);
    },

    async buttonReset(activity_id: string, button: string): Promise<Activity> {
      await API.buttonReset(activity_id, button);
      return this.getActivity(activity_id);
    },

    async allButtonsReset(activity_id: string): Promise<Activity> {
      await API.allButtonsReset(activity_id);
      return this.getActivity(activity_id);
    },
  },
});
