import type { Store } from "pinia";
import { defineStore } from "pinia";
import ApiConnection from "@/api";
import Api from "@/api";
import type ServiceAuth from "@/api/services/authenticate";
import sStorageWrapper from "@/api/local";
import { LoginState } from "@/types/enums";
import { addError } from "@/stores/messages";

const PIN_KEY = "user.auth.response";
const stored = sStorageWrapper.getValue(PIN_KEY);

export type AuthState = {
  inited: boolean;
  authenticated: LoginState;
  error: string;
};
export type AuthGetters = {
  isAuthenticated: boolean;
};
export type AuthActions = {
  init: () => void;
  authenticate: (pin: string) => Promise<boolean>;
  logout: () => Promise<boolean>;
};
export type AuthStore = Store<"user", AuthState, AuthGetters, AuthActions>;

export const authStorage = defineStore({
  id: "user",
  state: () => ({
    inited: false,
    authenticated: LoginState.NOT_DEFINED,
    error: "",
  }),
  getters: {
    isAuthenticated: (state: AuthState) => {
      return state.authenticated === LoginState.AUTHORISED;
    },
  },
  actions: {
    init() {
      if (this.inited) {
        return;
      }
      this.inited = true;
      Api.rest().addErrorInterceptor(
        "auth",
        (error) => {
          if (!Api.rest().isUnauthorizedError(error)) {
            return;
          }
          Api.rest().abort();

          addError({
            id: "error.session_expired",
            translate: "error.session_expired",
          });
          this.logout();
        },
        true
      );
      if (stored) {
        this.authenticate("" + stored);
      }
    },

    async authenticate(pin: string) {
      if (this.$state.authenticated === LoginState.AUTHORISED) {
        return true;
      }

      const auth = ApiConnection.getService("auth") as ServiceAuth;

      const setLoggedIn = () => {
        this.$state.authenticated = LoginState.AUTHORISED;
        this.$state.error = "";
        sStorageWrapper.setValue(PIN_KEY, pin);
      };

      const setError = (code?: string) => {
        this.$state.authenticated = LoginState.ANONYMOUS;
        this.$state.error = code || "unknown";
        sStorageWrapper.clearValue(PIN_KEY);
      };

      let result = false;
      try {
        const resp = await auth.authenticate(pin);
        if (resp.result) {
          setLoggedIn();
          result = true;
        } else {
          setError(resp.code);
        }
      } catch (e) {
        console.error(e);
        let error = null;
        if (typeof (e as any)?.code === "string") {
          error = (e as any)?.code;
        } else if (typeof (e as any)?.message === "string") {
          error = (e as any)?.message;
        } else if (typeof e === "string") {
          error = e;
        }
        setError(error);
      }
      return result;
    },

    async logout() {
      if (this.$state.authenticated !== LoginState.AUTHORISED) {
        return true;
      }

      sStorageWrapper.clearValue(PIN_KEY);
      this.$state.authenticated = LoginState.ANONYMOUS;
      this.$state.error = "";
      const auth = ApiConnection.getService("auth") as ServiceAuth;
      try {
        await auth.logout();
        return true;
      } catch (e) {
        this.$state.error = e as string;
      }
      return false;
    },
  },
});
