import { defineStore } from "pinia";
import ApiConnection from "@/api";
import type ServiceConfig from "@/api/services/config";
import type {
  CfgAll,
  CountryListItem,
  LanguageListItem,
  MeasurementUnitList,
  ConfigLists,
} from "@/types/config";
import { SelectTypes } from "@/types/enums";
import type { DeviceButtonLayout } from "@/types/activity";
import type { SupportedResource } from "@/types/resources";

const API: ServiceConfig = ApiConnection.config as ServiceConfig;

function getList(lists: ConfigLists, type: SelectTypes) {
  if (type === SelectTypes.TimeZone) {
    return lists.tz;
  }
  if (type === SelectTypes.VoiceAssistant) {
    return lists.voiceAssistants;
  }
  if (type === SelectTypes.Language) {
    return lists.languages;
  }
  if (type === SelectTypes.Country) {
    return lists.countries;
  }
  if (type === SelectTypes.UnitSystem) {
    return lists.unitSystems;
  }
  return false;
}

export const configStore = defineStore({
  id: "config",
  state: () => ({
    config: null as CfgAll | null,
    error: null as string | null,
    updateError: null as string | null,
    deviceName: null as string | null | false,
    list: {
      tz: null as string[] | null,
      voiceAssistants: null as string[] | null,
      languages: null as LanguageListItem[] | null,
      countries: null as CountryListItem[] | null,
      unitSystems: null as MeasurementUnitList | null,
      buttonLayout: null as DeviceButtonLayout[] | null,
      resourceTypes: null as SupportedResource[] | null,
    } as ConfigLists,
  }),
  actions: {
    init() {
      ApiConnection.websocket().addMessageCallback("config", (data) => {
        if (data.msg === "configuration_change") {
          API.getAll().then((cfg) => {
            this.$state.config = cfg;
          });
        }
      });
    },

    async getDeviceName(update = false): Promise<string | null | false> {
      if (this.$state.deviceName === null || update) {
        this.$state.deviceName = await API.getDeviceName();
      }
      return this.$state.deviceName;
    },

    async getAll(): Promise<CfgAll | null> {
      try {
        const {
          cfg,
          tz,
          voiceAssistants,
          languages,
          countries,
          unitSystems,
          buttonLayout,
        } = await API.loadAll();
        let deviceName = this.$state.deviceName;
        if (cfg.device?.name) {
          deviceName = cfg.device.name;
        }
        this.$state = {
          ...this.$state,
          config: cfg,
          deviceName,
          list: {
            ...this.$state.list,
            tz,
            voiceAssistants,
            languages,
            countries,
            unitSystems,
            buttonLayout,
          },
        };
        return this.$state.config;
      } catch (e) {
        this.$state.error = e as string;
        return null;
      }
    },

    async update(
      group: string,
      name: string,
      value: unknown
    ): Promise<CfgAll | null> {
      try {
        const resp: CfgAll = await API.update(group, name, value);
        if (resp.device?.name) {
          this.$state.deviceName = resp.device.name;
        }
        return resp;
      } catch (e) {
        this.$state.updateError = e as string;
        return null;
      }
    },

    async reset(): Promise<CfgAll | null> {
      try {
        await API.reset();
        return this.getAll();
      } catch (e) {
        this.$state.updateError = e as string;
        return null;
      }
    },
  },
  getters: {
    options(state) {
      return (type: SelectTypes) => {
        return getList(state.list, type);
      };
    },
    optionValueLabel(state) {
      return (type: SelectTypes, value: string | number) => {
        const options: any = getList(state.list, type);
        console.log("TYPE", type, options);
        if (!options) {
          return null;
        }
        if (Array.isArray(options)) {
          const index = options.indexOf(value);
          if (index !== -1) {
            return value;
          }
        }
        return options[value] || null;
      };
    },
  },
});
