import { defineStore } from "pinia";
import type {
  DockConfiguration,
  DockDiscovery,
  DockDiscoveryChangeMessage,
  DockDiscoveryList,
  DockSetup,
  DockSetupChangeMessage,
  DockStateMessage,
  DockSetupInfo,
  DockUpdateCheck,
  DockUpdateProgressMessage,
} from "@/types/dock";
import {
  DockDiscoveryChangeEventType,
  DockState,
  DockSetupState,
  DockUpdateProgressEventState,
} from "@/types/enums";
import ApiConnection from "@/api";

import { addError } from "@/stores/messages";

const API = ApiConnection.docks;

export const docksStore = defineStore({
  id: "docks",
  state: () => ({
    inited: false,
    docks: [] as DockConfiguration[],
    dockUpdateList: [] as DockUpdateCheck[],
    discoverActive: false,
    discovered: {} as DockDiscoveryList,
  }),

  actions: {
    init() {
      if (this.$state.inited) {
        return;
      }

      ApiConnection.websocket().addMessageCallback("dock", (data) => {
        if (data.msg === "dock_update_change") {
          this.dockUpdateChange(
            (data.msg_data || {}) as DockUpdateProgressMessage
          );
        }
        if (data.msg === "dock_discovery") {
          this.discoveryChange(
            (data.msg_data || {}) as DockDiscoveryChangeMessage
          );
        }
        if (data.msg === "dock_setup_change") {
          this.setupChange((data.msg_data || {}) as DockSetupChangeMessage);
        }
        if (data.msg === "dock_state") {
          this.stateChange((data.msg_data || {}) as DockStateMessage);
        }
      });

      this.$state.inited = true;
    },

    async getDockList(update = false): Promise<DockConfiguration[]> {
      this.init();
      if (!this.$state.docks.length || update) {
        const docks = await API.getDockList();
        this.$state.docks = Object.keys(docks).map((dock_id) => {
          return docks[dock_id];
        });
      }
      return this.$state.docks;
    },

    async getDockUpdateList(): Promise<any[]> {
      const updates = <DockUpdateCheck[]>[];
      const docks = await this.getDockList();

      for (const dock of docks) {
        let statusData = await this.getUpdateStatus(dock.dock_id, true);
        if (statusData.update_available === true) {
          updates.push({
            ...statusData,
            dock_configuration: dock,
          })
        }
      }
      this.$state.dockUpdateList = updates;
      return this.$state.dockUpdateList;
    },

    async getDockBrightness(dock_id: string): Promise<number> {
      return API.getDockBrightness(dock_id);
    },

    async setDockBrightness(dock_id: string, value: number): Promise<boolean> {
      return API.setDockBrightness(dock_id, value);
    },

    async changeDockName(
      dock_id: string,
      name: string
    ): Promise<DockConfiguration> {
      const dock = await API.updateDock(dock_id, {
        name,
      });
      const index = this.$state.docks.findIndex((item) => {
        return item.dock_id === dock.dock_id;
      });
      this.$state.docks[index] = dock;
      return dock;
    },

    async changeDockUrl(
      dock_id: string,
      url: string
    ): Promise<DockConfiguration> {
      const dock = await API.updateDock(dock_id, {
        custom_ws_url: url,
      });
      const index = this.$state.docks.findIndex((item) => {
        return item.dock_id === dock.dock_id;
      });
      this.$state.docks[index] = dock;
      return dock;
    },

    async identifyDock(dock_id: string): Promise<boolean> {
      return API.identifyDock(dock_id);
    },

    async changePass(dock_id: string, newPass: string) {
      const dock = await API.updateDock(dock_id, {
        token: newPass,
      });
      const index = this.$state.docks.findIndex((item) => {
        return item.dock_id === dock.dock_id;
      });
      this.$state.docks[index] = dock;
    },

    async changeWifi(dock_id: string, ssid: string, password: string) {
      const dock = await API.updateDock(dock_id, {
        wifi: {
          ssid,
          password,
        },
      });
      const index = this.$state.docks.findIndex((item) => {
        return item.dock_id === dock.dock_id;
      });
      this.$state.docks[index] = dock;
    },

    async factoryReset(dock_id: string): Promise<boolean> {
      const resp = await API.factoryReset(dock_id);
      await this.getDockList(true);
      return resp;
    },

    async removeDock(dock_id: string): Promise<DockConfiguration[]> {
      await API.removeDock(dock_id);
      return this.getDockList(true);
    },

    async getUpdateStatus(dock_id: string, forcedCheck: boolean = false): Promise<DockUpdateCheck> {
      return API.getUpdateStatus(dock_id, forcedCheck);
    },

    async startUpgrade(dock_id: string): Promise<boolean> {
      this.init();
      return API.startUpgrade(dock_id);
    },

    async abortUpgrade(dock_id: string): Promise<boolean> {
      return API.abortUpgrade(dock_id);
    },

    async dockUpdateChange(msg_data: DockUpdateProgressMessage) {
      // This is also a placeholder action to make possible to subscribe on.
      if (
        msg_data.state === DockUpdateProgressEventState.OK ||
        msg_data.state === DockUpdateProgressEventState.ERROR
      ) {
        this.getDockList(true);
      }
      return true;
    },

    async startDiscovery(): Promise<boolean> {
      if (this.$state.discoverActive) {
        return true;
      }

      try {
        const result = await API.startDiscovery();
        this.$state.discoverActive = true;
        this.$state.discovered = {};
        return result;
      } catch (e: any) {
        throw new Error(e.message || e);
      }
    },

    async stopDiscovery(): Promise<boolean> {
      try {
        this.$state.discoverActive = false;
        this.$state.discovered = {};
        return API.stopDiscovery();
      } catch (e: any) {
        throw new Error(e.message || e);
      }
    },

    async discoveryChange(msg_data: DockDiscoveryChangeMessage) {
      if (msg_data.event_type === DockDiscoveryChangeEventType.START) {
        await this.getDiscoveryStatus();
      }
      if (msg_data.event_type === DockDiscoveryChangeEventType.STOP) {
        this.$state.discoverActive = false;
      }
      if (
        this.$state.discoverActive &&
        msg_data.event_type === DockDiscoveryChangeEventType.DISCOVERY &&
        msg_data.dock
      ) {
        this.$state.discovered[msg_data.dock.id] = msg_data.dock;
      }
      return {
        active: this.$state.discoverActive,
        discovered: this.$state.discovered,
      };
    },

    async getDiscoveryStatus() {
      const result = await API.getDiscoveryStatus();
      this.$state.discoverActive = result.active;
      this.$state.discovered = result.discovered;
      return {
        active: this.$state.discoverActive,
        discovered: this.$state.discovered,
      };
    },

    async startSetupDiscoveredSetup(
      discovery: DockDiscovery,
      dock_setup: DockSetup
    ): Promise<DockSetupInfo> {
      const result = await API.startSetupDiscoveredSetup(discovery);
      await this.startSetupProcess(discovery.id, dock_setup);
      return result;
    },

    async startSetupManualSetup(dock: DockSetup): Promise<DockSetupInfo> {
      return API.startSetupManualSetup(dock);
    },

    async setupChange(msg_data: DockSetupChangeMessage) {
      if (
        msg_data.state === DockSetupState.OK ||
        msg_data.state === DockSetupState.ERROR
      ) {
        await this.getDockList(true);
      }

      return true;
    },

    async stateChange(msg_data: DockStateMessage) {
      const index = this.$state.docks.findIndex((item) => {
        return item.dock_id === msg_data.dock_id;
      });

      if (index > -1) {
        this.$state.docks[index].state = (msg_data.state || '') as DockState;
      }

      return true;
    },

    async startSetupProcess(
      dock_id: string,
      dock_setup: DockSetup
    ): Promise<DockSetupInfo> {
      return API.startSetupProcess(dock_id, dock_setup);
    },

    async cancelSetup(dock_id: string): Promise<boolean> {
      return API.cancelSetup(dock_id);
    },
  },
});
