import { defineStore } from "pinia";
import type {
  AvailableEntity,
  ConfiguredEntity,
  DriverConnectionTestResult,
  DriverId,
  EntityRequest,
  IntegrationDiscoveryChangeMessage,
  IntegrationDriver,
  IntegrationDriverInfo,
  IntegrationDriverUpdate,
  IntegrationInstance,
  IntegrationRequest,
  IntegrationSetupChangeMessage,
  IntegrationSetupData,
  IntegrationSetupDataInput,
  IntegrationSetupInfo,
  IntegrationStatus,
  IntegrationStateMessage,
  NewIntegrationData,
} from "@/types/integrationInstance";
import ApiConnection from "@/api";
import type { EntityCommandMetadata } from "@/types/activity";
import {
  DeviceState,
  DriverType,
  IntegrationState,
  IntegrationDiscoveryChangeEventType,
  IntegrationSetupState,
} from "@/types/enums";

const API = ApiConnection.integrations;

export const integrationsStore = defineStore({
  id: "integrations",
  state: () => ({
    inited: false,
    statuses: [] as IntegrationStatus[],
    instances: [] as IntegrationInstance[],
    enabledDrivers: [] as IntegrationDriverInfo[],
    instantiableDrivers: [] as IntegrationDriverInfo[],
    notConfiguredDrivers: [] as IntegrationDriverInfo[],
    availableEntities: [] as AvailableEntity[],
    configuredIntegrationEntities: [] as ConfiguredEntity[],
    configuredEntities: [] as ConfiguredEntity[],
    commands: [] as EntityCommandMetadata[],
    discoverActive: false,
    discovered: [] as IntegrationDriverInfo[],
  }),

  actions: {
    init() {
      if (this.inited) {
        return;
      }

      this.$state.inited = true;
      ApiConnection.websocket().addMessageCallback("integrations", (data) => {
        if (data.msg === "integration_driver_change") {
          // show newly created integration driver in overview screen.
          // The integration setup process might be cancelled: the driver still exists though!
          if (data.msg_data && data.msg_data.event_type === "NEW") {
            // The following data requests were just a best guess what to load to fix #85 :-)
            const requests = [
              this.getStatuses(true),
              this.getInstantiableDrivers(true),
              this.getNotConfiguredDrivers(true),
            ];
            Promise.all(requests).then(() => {
              // no idea what this is for...
              this.socketUpdate(data.msg_data || {});
            });
          }
        } else if (data.msg === "integration_change") {
          const requests = [
            this.getStatuses(true),
            this.getInstances(true),
            this.getInstantiableDrivers(true),
            this.getCommandMetadata(true),
            this.getNotConfiguredDrivers(true),
          ];
          Promise.all(requests).then(() => {
            this.socketUpdate(data.msg_data || {});
          });
        } else if (data.msg === "integration_discovery") {
          this.integrationDiscovery(
            (data.msg_data || {}) as IntegrationDiscoveryChangeMessage
          );
        } else if (data.msg === "integration_setup_change") {
          this.integrationSetupChange(
            (data.msg_data || {}) as IntegrationSetupChangeMessage
          );
        } else if (data.msg === "entity_change") {
          this.getConfiguredEntities(null, true);
        } else if (data.msg === "integration_state") {
          this.stateChange(
            (data.msg_data || {}) as IntegrationStateMessage
          );
        }
      });
    },

    async socketUpdate(msg_data: Record<string, any>) {
      // This is only a placeholder action to make possible to subscribe on.
    },

    async getStatuses(update = false): Promise<IntegrationStatus[]> {
      if (!this.$state.statuses || !this.$state.statuses.length || update) {
        try {
          const items = await API.getIntegrationStatuses();
          this.$state.statuses = items;
          return items;
        } catch (e) {
          this.$state.statuses = [];
        }
      }
      return this.$state.statuses;
    },

    async getInstances(update = false): Promise<IntegrationInstance[]> {
      if (!this.$state.instances || !this.$state.instances.length || update) {
        try {
          const items = await API.getInstances();
          this.$state.instances = items;
          return items;
        } catch (e) {
          this.$state.instances = [];
        }
      }
      return this.$state.instances;
    },

    async getEnabledDrivers(
      update = false
    ): Promise<IntegrationDriverInfo[]> {
      if (this.$state.discoverActive) {
        return this.$state.enabledDrivers;
      }

      if (
        !this.$state.enabledDrivers ||
        !this.$state.enabledDrivers.length ||
        update
      ) {
        try {
          const items = await API.getEnabledDrivers();
          this.$state.enabledDrivers = items;
          return items;
        } catch (e) {
          this.$state.enabledDrivers = [];
        }
      }
      return this.$state.enabledDrivers;
    },

    async getInstantiableDrivers(
      update = false
    ): Promise<IntegrationDriverInfo[]> {
      if (this.$state.discoverActive) {
        return this.$state.instantiableDrivers;
      }

      if (
        !this.$state.instantiableDrivers ||
        !this.$state.instantiableDrivers.length ||
        update
      ) {
        try {
          const items = await API.getInstantiableDrivers();
          this.$state.instantiableDrivers = items;
          return items;
        } catch (e) {
          this.$state.instantiableDrivers = [];
        }
      }
      return this.$state.instantiableDrivers;
    },

    async getNotConfiguredDrivers(
      update = false
    ): Promise<IntegrationDriverInfo[]> {
      if (
        !this.$state.notConfiguredDrivers ||
        !this.$state.notConfiguredDrivers.length ||
        update
      ) {
        try {
          const items = await API.getNotConfiguredDrivers();
          this.$state.notConfiguredDrivers = items;
          return items;
        } catch (e) {
          this.$state.notConfiguredDrivers = [];
        }
      }
      return this.$state.notConfiguredDrivers;
    },

    async getCommandMetadata(update = false): Promise<EntityCommandMetadata[]> {
      if (!this.$state.commands || !this.$state.commands.length || update) {
        this.$state.commands = await ApiConnection.config.getCommandData(
          update
        );
      }
      return this.$state.commands;
    },

    async getEntities(
      integration_id: string,
      update = false,
      user_fetch_first_page = false
    ): Promise<AvailableEntity[]> {
      if (
        (!this.$state.availableEntities ||
        !this.$state.availableEntities.length ||
        update) && user_fetch_first_page == true
      ) {
        const paramFilter = "NEW"
        this.$state.availableEntities = await API.getEntities(integration_id, paramFilter, true);
      } else if (
        !this.$state.availableEntities ||
        !this.$state.availableEntities.length ||
        update
      ) {
        const paramFilter = "NEW"
        this.$state.availableEntities = await API.getEntities(integration_id, paramFilter);
      }
      return this.$state.availableEntities;
    },

    async getEntitiesByPage(
      integration_id: string,
      update = false,
      user_fetch_first_page = false,
      page: number = 1,
      searchText: string = '',
      entityTypes: string = '',
    ): Promise<{data: AvailableEntity[], headers: object}> {
      let newEntities = <{data: AvailableEntity[], headers?: object}>{}
      if (
        (!this.$state.availableEntities ||
        !this.$state.availableEntities.length ||
        update) && user_fetch_first_page == true
      ) {
        const paramFilter = "NEW"
        newEntities = await API.getEntitiesByPage(integration_id, paramFilter, true, page, searchText, entityTypes)
        this.$state.availableEntities = newEntities.data;
      } else if (
        !this.$state.availableEntities ||
        !this.$state.availableEntities.length ||
        update
      ) {
        const paramFilter = "NEW"
        newEntities = await API.getEntitiesByPage(integration_id, paramFilter, false, page, searchText, entityTypes)
        this.$state.availableEntities = this.$state.availableEntities.concat(newEntities.data);
      }
      return {data: this.$state.availableEntities, headers: newEntities.headers || {}};
    },

    async getConfiguredEntities(
      inst_id: string | null = null,
      update = false
    ): Promise<ConfiguredEntity[]> {
      if (
        !this.$state.configuredEntities ||
        !this.$state.configuredEntities.length ||
        update
      ) {
        this.$state.configuredEntities = await API.getConfiguredEntities(
          inst_id
        );
      }
      return this.$state.configuredEntities;
    },

    async getConfiguredEntitiesByPage(
      integration_id: string = '',
      update = false,
      user_fetch_first_page = false,
      page: number = 1,
      searchText: string = '',
      entityTypes: string = '',
    ): Promise<{data: ConfiguredEntity[], headers: object}> {

      let newEntities = <{data: ConfiguredEntity[], headers?: object}>{}
      if (
        (!this.$state.configuredIntegrationEntities ||
        !this.$state.configuredIntegrationEntities.length ||
        update) && user_fetch_first_page == true
      ) {
        newEntities = await API.getConfiguredEntitiesByPage(integration_id, true, page, searchText, entityTypes)
        this.$state.configuredIntegrationEntities = newEntities.data;
      } else if (
        !this.$state.configuredIntegrationEntities ||
        !this.$state.configuredIntegrationEntities.length ||
        update
      ) {
        newEntities = await API.getConfiguredEntitiesByPage(integration_id, false, page, searchText, entityTypes)
        this.$state.configuredIntegrationEntities = this.$state.configuredIntegrationEntities.concat(newEntities.data);
      }
      return {data: this.$state.configuredIntegrationEntities, headers: newEntities.headers || {}};
    },

    async stateChange(msg_data: IntegrationStateMessage) {
      const index = this.$state.statuses.findIndex((item) => {
        return item.integration_id === msg_data.integration_id;
      });

      if (index > -1) {
        // DeviceState is a subset of IntegrationState
        this.$state.statuses[index].state = (msg_data.device_state || DeviceState.UNKNOWN) as unknown as IntegrationState;
      }

      return true;
    },

    async addIntegration(
      driver_id: DriverId,
      data: NewIntegrationData
    ): Promise<IntegrationInstance> {
      return await API.addIntegration(driver_id, data);
    },

    async getIntegrationStatus(
      inst_id: string,
      update = false
    ): Promise<IntegrationStatus | undefined> {
      if (update || !this.$state.statuses || !this.$state.statuses.length) {
        await this.getStatuses(update);
      }
      return this.$state.statuses.find((intsg: IntegrationStatus) => {
        return intsg.integration_id === inst_id;
      });
    },

    async getIntegration(
      inst_id: string,
      update = false
    ): Promise<{
      inst: IntegrationInstance | null;
      status: IntegrationStatus | null;
      driver: IntegrationDriver | null;
      entities: {data: AvailableEntity[], headers: any};
      configured: {data: ConfiguredEntity[], headers: any};
      _error?: Record<string, any>;
    }> {
      let inst = null;
      let status = null;
      let driver = null;
      let entities: {data: AvailableEntity[], headers: object} = {data: [], headers: {}};
      let configured: {data: ConfiguredEntity[], headers: object} = {data: [], headers: {}};
      let hasError = false;
      const errors: Record<string, any> = {};
      try {
        inst = await API.getIntegration(inst_id);
      } catch (e) {
        hasError = true;
        errors.inst = e;
      }
      if (inst) {
        try {
          driver = await API.getDriver(inst.driver_id);
        } catch (e) {
          hasError = true;
          errors.driver = e;
        }
      }
      try {
        status = await this.getIntegrationStatus(inst_id, update);
      } catch (e) {
        hasError = true;
        errors.status = e;
      }
      if (status?.state === IntegrationState.ACTIVE) {
        try {
          entities = await this.getEntitiesByPage(inst_id, update, true);
        } catch (e) {
          hasError = true;
          errors.entities = e;
        }
      }
      try {
        configured = await this.getConfiguredEntitiesByPage(inst_id, update, true);
      } catch (e) {
        console.log("configured", e);
        // hasError = true;
        // errors.configured = e;
      }
      return {
        inst,
        status: status || null,
        driver,
        entities,
        configured,
        _error: hasError ? errors : undefined,
      };
    },

    async updateEntityLists(inst_id: string, user_fetch_first_page = false): Promise<{
      entities: AvailableEntity[];
      configured: ConfiguredEntity[];
    }> {
      const entitiesPromise = this.getEntities(inst_id, true, user_fetch_first_page);
      return entitiesPromise.then(async (entities: AvailableEntity[]) => {
        const configured = await this.getConfiguredEntities(inst_id, true);
        return {
          entities,
          configured,
        };
      });
    },

    async pagedUpdateAvailableEntityLists(inst_id: string, user_fetch_first_page = false, page: number = 1, searchText: string = '', entityTypes: string = ''): Promise<{data: AvailableEntity[], headers: any}> {
      const entities = await this.getEntitiesByPage(inst_id, true, user_fetch_first_page, page, searchText, entityTypes);
      return entities;
    },

    async pagedUpdateConfiguredEntityLists(inst_id: string, user_fetch_first_page = false, page: number = 1, searchText: string = '', entityTypes: string = ''): Promise<{data: ConfiguredEntity[], headers: any}> {
      const configured = await this.getConfiguredEntitiesByPage(inst_id, true, user_fetch_first_page, page, searchText, entityTypes);
      return configured;
    },

    async updateIntegration(
      inst_id: string,
      data: IntegrationRequest
    ): Promise<IntegrationInstance> {
      const inst = await API.updateIntegration(inst_id, data);
      this.getInstances(true);
      this.getStatuses(true);
      return inst;
    },

    async addInstanceEntity(
      inst: IntegrationInstance,
      entity: AvailableEntity
    ): Promise<ConfiguredEntity> {
      return await API.addInstanceEntity(inst, entity);
    },

    async addInstanceEntities(
      inst: IntegrationInstance,
      entity_ids: string[]
    ): Promise<string[]> {
      return await API.addInstanceEntities(inst, entity_ids);
    },

    async removeEntity(entity: ConfiguredEntity): Promise<boolean> {
      const response = await API.removeEntity(entity);
      return response.code === "OK";
    },

    async removeEntities(entity_ids: string[]): Promise<boolean> {
      const response = await API.removeEntities(entity_ids);
      return response.code === "OK";
    },

    async updateEntity(
      entity_id: string,
      changes: EntityRequest
    ): Promise<ConfiguredEntity> {
      const resp = await API.updateEntity(entity_id, changes);
      this.$state.configuredEntities = await this.getConfiguredEntities(
        null,
        true
      );
      if (resp) {
        return resp;
      }
      return this.$state.configuredEntities.find((entity: ConfiguredEntity) => {
        return entity.entity_id === entity_id;
      }) as ConfiguredEntity;
    },

    async connectInst(inst_id: string): Promise<{
      inst: IntegrationInstance;
      status: IntegrationStatus;
    }> {
      const inst = await API.connectInst(inst_id);
      const status = await this.getIntegrationStatus(inst_id, true);
      return {
        inst,
        status: status as IntegrationStatus,
      };
    },

    async disconnectInst(inst_id: string): Promise<{
      inst: IntegrationInstance;
      status: IntegrationStatus;
    }> {
      const inst = await API.disconnectInst(inst_id);
      const status = await this.getIntegrationStatus(inst_id, true);
      return {
        inst,
        status: status as IntegrationStatus,
      };
    },

    async deleteIntegration(integration: IntegrationStatus): Promise<boolean> {
      const resp = await API.deleteIntegration(integration);
      await Promise.all([
        this.getStatuses(true),
        this.getNotConfiguredDrivers(true),
      ]);
      return resp;
    },

    async updateDriver(
      driver_id: string,
      data: IntegrationDriverUpdate
    ): Promise<IntegrationDriver> {
      await API.updateDriver(driver_id, data);
      return API.getDriver(driver_id);
    },

    async startDiscovery(): Promise<boolean> {
      if (this.$state.discoverActive) {
        return true;
      }
      const result = await API.startDiscovery();
      this.$state.discoverActive = true;
      this.$state.discovered = [];
      return result;
    },

    async stopDiscovery(): Promise<boolean> {
      this.$state.discoverActive = false;
      this.$state.discovered = [];
      return API.stopDiscovery();
    },

    async integrationDiscovery(msg_data: IntegrationDiscoveryChangeMessage) {
      if (msg_data.event_type === IntegrationDiscoveryChangeEventType.START) {
        this.$state.discoverActive = true;
      } else if (
        msg_data.event_type === IntegrationDiscoveryChangeEventType.STOP
      ) {
        this.$state.discoverActive = false;
      } else if (
        this.$state.discoverActive &&
        msg_data.event_type === IntegrationDiscoveryChangeEventType.DISCOVERY &&
        msg_data.integration
      ) {
        this.$state.discovered.push({
          driver_id: msg_data.integration.id,
          name: {
            en: msg_data.integration.name,
          },
          driver_type: DriverType.EXTERNAL,
          driver_url: msg_data.integration.driver_url,
          pwd_protected: msg_data.integration.pwd_protected,
          version: msg_data.integration.version || "",
          icon: msg_data.integration.icon,
          developer_name: msg_data.integration.developer_name,
          discovered: true,
        });
      }
    },

    async testConnection(
      driver_id: string,
      driver_url: string,
      token: string | undefined = undefined
    ): Promise<DriverConnectionTestResult> {
      return API.testConnection(driver_id, driver_url, token);
    },

    async registerIntegration(
      driver_id: string,
      driver_url: string,
      token: string | undefined = undefined
    ): Promise<IntegrationDriver> {
      return API.registerIntegration(driver_id, driver_url, token);
    },

    async getDriver(driver_id: string): Promise<IntegrationDriver> {
      return API.getDriver(driver_id);
    },

    async startSetupIntegration(
      driver_id: string,
      setup_data: IntegrationSetupDataInput
    ): Promise<IntegrationSetupInfo> {
      return API.startSetupIntegration(driver_id, setup_data);
    },

    async continueSetupIntegration(
      driver_id: string,
      setup_data: IntegrationSetupData
    ): Promise<IntegrationSetupInfo> {
      return API.continueSetupIntegration(driver_id, setup_data);
    },

    async integrationSetupChange(data_msg: IntegrationSetupChangeMessage) {
      if (data_msg.state === IntegrationSetupState.OK) {
        await this.getInstances(true);
      }
    },

    async stopSetupIntegration(driver_id: string): Promise<boolean> {
      const response = await API.stopSetupIntegration(driver_id);
      await this.getInstances(true);
      return response;
    },
  },
});
