import { defineStore } from "pinia";
import type { Remote, RemoteNewData, RemoteUpdate } from "@/types/remote";
import type { ActivityUserInterfacePage } from "@/types/activity";
import type { IrCodeDefinition, RemoteDataSet, RemoteIrCode } from "@/types/ir";

import ApiConnection from "@/api";

const API = ApiConnection.remotes;

export const remotesStore = defineStore({
  id: "remotes",
  state: () => ({
    inited: false,
    remotes: [] as Remote[] | [],
    pages: [] as ActivityUserInterfacePage[] | [],
  }),

  actions: {
    init() {
      if (this.$state.inited) {
        return;
      }
      this.$state.inited = true;
      ApiConnection.websocket().addMessageCallback("remotes", (data) => {
        if (
          data.msg === "entity_change" &&
          data.msg_data?.new_state?.entity_type === "remote"
        ) {
          this.getAll();
        }
        this.socketUpdate(data.msg_data || {});
      });
    },

    async getAll(): Promise<Remote[]> {
      this.init();
      const items = await API.getAll();
      this.$state.remotes = items;
      return items;
    },

    async socketUpdate(msg_data: Record<string, any>) {
      // This is only a placeholder action to make possible to subscribe on.
    },

    async getRemote(entity_id: string): Promise<Remote> {
      this.init();
      return await API.getRemote(entity_id);
    },

    async getRemoteIrCodes(entity_id: string): Promise<RemoteDataSet> {
      return await API.getRemoteIrCodes(entity_id);
    },

    async addCustomCodeToSet(
      entity_id: string,
      cmd_id: string,
      code: IrCodeDefinition
    ): Promise<RemoteIrCode> {
      return await API.addCustomCodeToSet(entity_id, cmd_id, code);
    },

    async removeCustomCodeFromSet(
      entity_id: string,
      cmd_id: string
    ): Promise<boolean> {
      return await API.removeCustomCodeFromSet(entity_id, cmd_id);
    },

    async create(remoteData: RemoteNewData): Promise<Remote> {
      return await API.createNewRemote(remoteData);
    },

    async update(entity_id: string, data: RemoteUpdate): Promise<Remote> {
      await API.update(entity_id, data);
      return this.getRemote(entity_id);
    },

    async delete(remote: Remote): Promise<Remote[]> {
      await API.delete(remote);
      return this.getAll();
    },

    async addUiPage(remote: Remote, name: string) {
      await API.addUiPage(remote, name);
      return this.getRemote(remote.entity_id);
    },

    async deleteUiPage(remote_id: string, page_id: string) {
      await API.deleteUiPage(remote_id, page_id);
      return this.getRemote(remote_id);
    },

    async updateUiPage(remote_id: string, page: ActivityUserInterfacePage) {
      await API.updateUiPage(remote_id, page);
      return this.getRemote(remote_id);
    },

    async allUiReset(remote_id: string): Promise<Remote> {
      await API.allUiReset(remote_id);
      return this.getRemote(remote_id);
    },

    async updatePagesOrder(
      remote: Remote,
      pageValues: ActivityUserInterfacePage[]
    ) {
      const pages: string[] = Array.from(pageValues).map((page) => {
        return page.page_id;
      });
      await API.updateUiPageOrder(remote, pages);

      return this.getPages(remote);
    },

    async getPages(remote: Remote) {
      this.$state.pages = await API.getPages(remote);
      return this.$state.pages;
    },

    async buttonUpdate(
      remote_id: string,
      button: string,
      command: string
    ): Promise<Remote> {
      await API.buttonUpdate(remote_id, button, command);
      return this.getRemote(remote_id);
    },

    async buttonReset(remote_id: string, button: string): Promise<Remote> {
      await API.buttonReset(remote_id, button);
      return this.getRemote(remote_id);
    },

    async allButtonsReset(remote_id: string): Promise<Remote> {
      await API.allButtonsReset(remote_id);
      return this.getRemote(remote_id);
    },
  },
});
