import type {
  DeviceButtonGroup,
  EntityType,
  SequenceType,
} from "@/types/enums";
import type { CommandSequence, CommandSequenceDelay } from "@/types/command";
import type { IconIdentifier, LanguageText } from "@/types/config";

export type ActivityNewData = {
  name: LanguageText;
  icon?: string;
  description?: LanguageText;
  options?: {
    entity_ids?: string[];
  };
};

export type IncludedEntity = {
  entity_id: string;
  entity_type?: EntityType;
  name?: LanguageText;
  icon?: IconIdentifier;
  integration_id?: string;
  integration?: {
    name?: LanguageText;
    icon?: IconIdentifier;
  };
  entity_commands?: string[];
  simple_commands?: string[];
  available?: boolean;
};

export type Activity = {
  entity_id: string;
  entity_type: EntityType.activity;
  integration_id: string;
  device_class?: string;
  name: LanguageText;
  description?: LanguageText;
  icon?: string;
  features?: "on_off" | "start";
  options?: {
    editable: boolean;
    sequences: ActivitySequences;
    button_mapping: DeviceButtonMapping[];
    user_interface: ActivityUserInterface;
    included_entities: IncludedEntity[];
  };
};

export type ActivityUpdate = {
  name?: LanguageText;
  description?: LanguageText;
  icon?: IconIdentifier;
  options?: {
    entity_ids?: string[];
    sequences?: ActivitySequences;
    button_mapping?: DeviceButtonMapping[];
    user_interface?: ActivityUserInterface;
  };
};

export type ActivitySequences = {
  on: CommandSequence[];
  off: CommandSequence[];
};

export type ActivityUserInterface = {
  pages: ActivityUserInterfacePage[];
};
export type ActivityUserInterfacePage = {
  page_id: string;
  name?: string;
  items: ActivityUserInterfaceItem[];
};
export type ActivityUserInterfaceGridItem = {
  x: number;
  y: number;
  w: number;
  h: number;
  i: string;
  item: ActivityUserInterfaceItem;
};
export type ActivityUserInterfaceItem = {
  type: "icon" | "text" | "numpad";
  icon?: IconIdentifier;
  text?: string;
  command?: EntityCommand;
  short_press?: EntityCommand;
  location: GridLocation;
  size: GridItemSize;
};

export type EntityCommand = {
  entity_id?: string;
  cmd_id?: string;
  params?: CommandParameter;
};

// I'm a big fan of collections, but somehow the Map was never serialized in the json payload in services/activities:update
// export type CommandParameter = Map<string, any>;
// Therefore, back a plain object
export type CommandParameter = {
  [key: string]: any;
};

export type CommandItem = (EntityCommandListItem | DelayCommandListItem) & {
  command_type: "delay" | "simple" | "entity" | null;
};

export type DelayCommandListItem = {
  type: SequenceType.delay;
  cmd: CommandSequenceDelay;
};

export type EntityCommandListItem = {
  type: SequenceType.command;
  cmd: EntityCommandMetadata;
  id?: string;
  entity?: IncludedEntity;
};

export type CommandSequenceListItem = {
  command_type: "delay" | "simple" | "entity" | null;
  pos: number;
  list?: string;
  id?: string;
  type: SequenceType;
  cmd?: EntityCommandMetadata;
  entity?: IncludedEntity;
  sequence: CommandSequence;
};

export type EntityCommandMetadata = {
  id: string;
  cmd_id: string;
  name: LanguageText;
  params?: EntityCmdParam[] | CommandParameter;
  entity_id?: string;
  entity_name?: LanguageText;
};

export type EntityCmdParam =
  | EntityCmdParamNumber
  | EntityCmdParamBool
  | EntityCmdParamRegex
  | EntityCmdParamEnum;

type EntityCommandMetadataParamBase = {
  name: LanguageText;
  param: string;
  type: "number" | "bool" | "regex" | "enum";
};

export type EntityCmdParamNumber = EntityCommandMetadataParamBase & {
  min?: number;
  max?: number;
  step?: number;
  value?: number;
  unit?: string;
};
export type EntityCmdParamBool = EntityCommandMetadataParamBase;
export type EntityCmdParamRegex = EntityCommandMetadataParamBase & {
  regex?: string;
};
export type EntityCmdParamEnum = EntityCommandMetadataParamBase & {
  value?: number;
  values: string[];
};

export type DeviceButtonLayout = {
  type: DeviceButtonGroup;
  grid: {
    width: number;
    height: number;
  };
  buttons: DeviceButton[];
};

export type DeviceButton = {
  button: string;
  icon: IconIdentifier;
  name: LanguageText;
  location: GridLocation;
  size: GridItemSize;
};

export type DeviceButtonMapping = {
  button: string;
  short_press?: EntityCommand;
  long_press?: EntityCommand;
};

export type GridLocation = {
  x: number;
  y: number;
};

export type GridItemSize = {
  width: number;
  height: number;
};
